#include "nvclock.h"
#include "xf86i2c.h"

#define _LIBASUS_H

#define LIBASUS_VERSION "0.1"

#define true 	1
#define false 	0

extern int	debug;

#define dbg_printf(x...) if (debug==1) printf(x)

#define LIBASUS_DEVICE_F75375	0x01

I2CDevPtr gfxmon;
I2CDevPtr init_asusfan();

typedef struct _fan_vtemp {
	int	temp[4];

	int	speed[5];
} fan_vtemp;

typedef struct _libasus_card {
	I2CDevPtr	card;

	// Generic temperature functions
	int	(*get_gpu_temp)(I2CDevPtr card);
	int	(*get_ram_temp)(I2CDevPtr card);
	int	(*get_gpu_fanspeed)(I2CDevPtr card);
	int	(*set_gpu_fanspeed)(I2CDevPtr dev, int desired_rpm);
	
	// F75375 specific
	int	(*get_gpu_fanmode)(I2CDevPtr dev);
	int	(*set_gpu_fanmode)(I2CDevPtr dev, I2CByte mode);
	int	(*get_gpu_tempctl)(I2CDevPtr dev, fan_vtemp *speeds);
	int	(*set_gpu_tempctl)(I2CDevPtr dev, fan_vtemp speeds);
	int	(*set_gpu_pwmspeed)(I2CDevPtr dev, int speed);
	int	(*get_gpu_pwmspeed)(I2CDevPtr dev);
} libasus_card;

// Chip header files here
#include "f75375.h"

int libasus_get_gpu_temp(libasus_card card);
int libasus_get_ram_temp(libasus_card card);
int libasus_get_gpu_fanspeed(libasus_card card);
int libasus_set_gpu_fanspeed(libasus_card card, int desired_rpm);
int libasus_get_gpu_fanmode(libasus_card card);
int libasus_set_gpu_fanmode(libasus_card card, I2CByte mode);
int libasus_set_gpu_pwmspeed(libasus_card card, int speed);
int libasus_get_gpu_pwmspeed(libasus_card card);
int libasus_set_gpu_tempctl(libasus_card card, fan_vtemp speeds);
int libasus_get_gpu_tempctl(libasus_card card, fan_vtemp *speeds);

int libasus_init(int enable_debug);
int libasus_detect(I2CDevPtr *card);
int libasus_create_card(libasus_card *card, I2CDevPtr i2cdev, int type);
