#include <sys/mman.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>

#include "nvclock.h"
#include "libnvidia.h"
#include "libasus.h"

card_list gfxcard;
Card nv_card;

int IsVideoCard(unsigned short devbusfn);

int map_mem(const char* dev_name);
void unmap_mem();
void *map_dev_mem (int fd, unsigned long Base, unsigned long Size);
void unmap_dev_mem (unsigned long Base, unsigned long Size);
int probe_devices();

int init_nvclock() {
  // FIXME: You should do a lot more here, like check for drivers etc

  if (!probe_devices()) {
    return 0;
  }

  return 1;
}

int probe_devices() {
  int dev, dum, reg_addr, i=0;
  unsigned short devbusfn;
  char buf[256];
  FILE *proc;

  proc = fopen("/proc/bus/pci/devices", "r");
  if (!proc) {
    // DO SOMETHING NASTY HERE
    return 0;
  }
	
  while (fgets(buf, sizeof(buf)-1, proc)) {
    if (sscanf(buf, "%hx %x %x %x", &devbusfn, &dev, &dum, &reg_addr) != 4) continue;

    if ((dev>>16) == 0x10de) {
      if (!IsVideoCard(devbusfn))
	continue;

      dbg_printf("Found Nvidia Card!\n");

      gfxcard.device_id = (0x0000ffff & dev);
      switch (gfxcard.device_id & 0x0ff0) {
	case 0x40:
        case 0x90:
        case 0xc0:
        case 0x120:
        case 0x130:
        case 0x140:
        case 0x160:
        case 0x1d0:
        case 0x210:
        case 0x220:
        case 0x230:
	  dbg_printf("Found some NV40 (6800) based card!\n");
          gfxcard.arch = NV40;
	  break;
        case 0x340:
          dbg_printf("Found an NV30-based card\n");
          gfxcard.arch = NV30;
          break;
	default:
	  dbg_printf("Found an unknown card (device ID 0x%x)\n",gfxcard.device_id);
	  return 0;
      }
      gfxcard.number = 0;
      gfxcard.devbusfn = devbusfn;

      switch (check_driver()) {
	case 0:
	  gfxcard.dev_name = strdup("/dev/mem");
	  gfxcard.reg_address = reg_addr;
	  break;
	case 1:
	  gfxcard.dev_name = strdup("/dev/nvidia0");
	  gfxcard.reg_address = 0;
	  break;
	case 2:
	  gfxcard.dev_name = strdup("/dev/nvidia0");
	  gfxcard.reg_address = reg_addr;
	  break;
      }
      
    }
  }
  fclose(proc);

  return 1;
}

// This is here called set_card to make sure its compat with nvclock. Hopefully I'll
// get the damned thing to be compatiable easily.
int set_card(int number) {
//  nv_card.card_name = gfxcard.card_name;
  nv_card.number = 0;
  nv_card.device_id = gfxcard.device_id;
  nv_card.arch = gfxcard.arch;
  nv_card.devbusfn = gfxcard.devbusfn;
  nv_card.reg_address = gfxcard.reg_address;

  if (!map_mem(gfxcard.dev_name))
    return 0;

  return 1;
}


int IsVideoCard(unsigned short devbusfn) {
  int32_t pci_class = pciReadLong(devbusfn, 0x09);
  if (((htonl(pci_class) >> 8) & 0x0f) != 0x03)
    return 0;
  else
    return 1;
}

int check_driver() {
  FILE *proc;
  char buffer[80];

  proc = fopen("/proc/modules", "r");

  if (proc != NULL) {
    while (fgets(buffer, 80, proc) != NULL) {
      if (strstr(buffer, "NVdriver") != 0) {
	fclose(proc);
	return 1;
      }

      if (strstr(buffer, "nvidia") != 0) {
	fclose(proc);
	return 2;
      }
    }
  }
  fclose(proc);

  return 0;
}

int32_t pciReadLong(unsigned short devbusfn, long offset)
{
    char file[25];
    FILE *device;
    short bus = PCI_GET_BUS(devbusfn);
    short dev = PCI_GET_DEVICE(devbusfn);
    short function = PCI_GET_FUNCTION(devbusfn);

    snprintf(file, sizeof(file), "/proc/bus/pci/%02x/%02x.%x", bus, dev, function);
    if((device = fopen(file, "r")) != NULL)
    {
        int32_t buffer;
        fseek(device, offset, SEEK_SET);
        fread(&buffer, sizeof(int32_t), 1, device);
        fclose(device);

        return buffer;
    }

    return -1;
}

// YOINK!
int map_mem(const char *dev_name)
{
    int fd;

    if( (fd = open(dev_name, O_RDWR)) == -1 )
    {
	char err[80];
	printf(err, "Can't open %s", dev_name);
	//set_error_str(err);
	return 0;
    }
    
    /* Map the registers of the nVidia chip */
    nv_card.PEXTDEV = map_dev_mem(fd, nv_card.reg_address + 0x101000, 0x1000);
    nv_card.PFB     = map_dev_mem(fd, nv_card.reg_address + 0x100000, 0x1000);
    /* normally pmc is till 0x2000 but extend it to behind 0x4000 for nv40 */
    nv_card.PMC     = map_dev_mem(fd, nv_card.reg_address + 0x000000, 0x4040);
    nv_card.PCIO    = map_dev_mem(fd, nv_card.reg_address + 0x601000, 0x2000);
    nv_card.PRAMDAC = map_dev_mem(fd, nv_card.reg_address + 0x680000, 0x2000);
    nv_card.PROM    = map_dev_mem(fd, nv_card.reg_address + 0x300000, 0xffff);
    
    return 1;
}

void unmap_mem()
{
    unmap_dev_mem((unsigned long)nv_card.PEXTDEV, 0x1000);
    unmap_dev_mem((unsigned long)nv_card.PFB, 0x1000);
    unmap_dev_mem((unsigned long)nv_card.PMC, 0x4040);
    unmap_dev_mem((unsigned long)nv_card.PRAMDAC, 0x2000);
    unmap_dev_mem((unsigned long)nv_card.PROM, 0xffff);
}

/* -------- mmap on devices -------- */
/* This piece of code is from nvtv a linux program for tvout */
/* The author of nvtv got this from xfree86's os-support/linux/lnx_video.c */
/* and he modified it a little  */
void *map_dev_mem (int fd, unsigned long Base, unsigned long Size)
{
    void *base;
    int mapflags = MAP_SHARED;
    unsigned long realBase, alignOff;

    realBase = Base & ~(getpagesize() - 1);
    alignOff = Base - realBase;

    base = mmap((caddr_t)0, Size + alignOff, PROT_READ|PROT_WRITE,
	mapflags, fd, (off_t)realBase);
    return (void *) ((char *)base + alignOff);
}

void unmap_dev_mem (unsigned long Base, unsigned long Size)
{
    unsigned long alignOff = Base - (Base & ~(getpagesize() - 1));
    munmap((caddr_t)(Base - alignOff), (Size + alignOff));
}
